@@ -1,17 +1,17 @@
+use crate::scalar::Scalar;
+use crate::CubicBezierSegment;
 ///! Computes intersection parameters for two cubic bézier curves using bézier clipping, also known
 ///! as fat line clipping.
 ///!
 ///! The implementation here was originally ported from that of paper.js:
 ///! https://github.com/paperjs/paper.js/blob/0deddebb2c83ea2a0c848f7c8ba5e22fa7562a4e/src/path/Curve.js#L2008
-///! See "Bézier Clipping method" in
+///! See "bézier Clipping method" in
 ///! https://scholarsarchive.byu.edu/facpub/1/
 ///! for motivation and details of how the process works.
-
-use crate::generic_math::{point, Point, Rect};
-use crate::scalar::Scalar;
-use crate::CubicBezierSegment;
+use crate::{point, Box2D, Point};
 use arrayvec::ArrayVec;
-use std::ops::Range;
+use core::mem;
+use core::ops::Range;
 
 // Computes the intersections (if any) between two cubic bézier curves in the form of the `t`
 // parameters of each intersection point along the curves.
@@ -23,8 +23,10 @@ use std::ops::Range;
 pub fn cubic_bezier_intersections_t<S: Scalar>(
     curve1: &CubicBezierSegment<S>,
     curve2: &CubicBezierSegment<S>,
-) -> ArrayVec<[(S, S); 9]> {
-    if !curve1.fast_bounding_rect().intersects(&curve2.fast_bounding_rect())
+) -> ArrayVec<(S, S), 9> {
+    if !curve1
+        .fast_bounding_box()
+        .intersects(&curve2.fast_bounding_box())
         || curve1 == curve2
         || (curve1.from == curve2.to
             && curve1.ctrl1 == curve2.ctrl2
@@ -38,14 +40,17 @@ pub fn cubic_bezier_intersections_t<S: Scalar>(
 
     #[inline]
     fn midpoint<S: Scalar>(point1: &Point<S>, point2: &Point<S>) -> Point<S> {
-        point((point1.x + point2.x) * S::HALF, (point1.y + point2.y) * S::HALF)
+        point(
+            (point1.x + point2.x) * S::HALF,
+            (point1.y + point2.y) * S::HALF,
+        )
     }
 
     let curve1_is_a_point = curve1.is_a_point(S::EPSILON);
     let curve2_is_a_point = curve2.is_a_point(S::EPSILON);
     if curve1_is_a_point && !curve2_is_a_point {
         let point1 = midpoint(&curve1.from, &curve1.to);
-        let curve_params = point_curve_intersections(&point1, &curve2, S::EPSILON);
+        let curve_params = point_curve_intersections(&point1, curve2, S::EPSILON);
         for t in curve_params {
             if t > S::EPSILON && t < S::ONE - S::EPSILON {
                 result.push((S::ZERO, t));
@@ -53,7 +58,7 @@ pub fn cubic_bezier_intersections_t<S: Scalar>(
         }
     } else if !curve1_is_a_point && curve2_is_a_point {
         let point2 = midpoint(&curve2.from, &curve2.to);
-        let curve_params = point_curve_intersections(&point2, &curve1, S::EPSILON);
+        let curve_params = point_curve_intersections(&point2, curve1, S::EPSILON);
         for t in curve_params {
             if t > S::EPSILON && t < S::ONE - S::EPSILON {
                 result.push((t, S::ZERO));
@@ -76,9 +81,16 @@ pub fn cubic_bezier_intersections_t<S: Scalar>(
         result = line_line_intersections(curve1, curve2);
     } else {
         add_curve_intersections(
-            &curve1, &curve2, &(S::ZERO..S::ONE), &(S::ZERO..S::ONE),
-            &mut result, /* flip */ false, /* recursion_count */ 0, /* call_count */ 0,
-            /* original curve1 */ &curve1, /* original curve2 */ &curve2,
+            curve1,
+            curve2,
+            &(S::ZERO..S::ONE),
+            &(S::ZERO..S::ONE),
+            &mut result,
+            /* flip */ false,
+            /* recursion_count */ 0,
+            /* call_count */ 0,
+            /* original curve1 */ curve1,
+            /* original curve2 */ curve2,
         );
     }
 
@@ -89,7 +101,7 @@ fn point_curve_intersections<S: Scalar>(
     pt: &Point<S>,
     curve: &CubicBezierSegment<S>,
     epsilon: S,
-) -> ArrayVec<[S; 9]> {
+) -> ArrayVec<S, 9> {
     let mut result = ArrayVec::new();
 
     // (If both endpoints are epsilon close, we only return S::ZERO.)
@@ -128,7 +140,7 @@ fn point_curve_intersections<S: Scalar>(
         }
     }
 
-    if result.len() > 0 {
+    if !result.is_empty() {
         return result;
     }
 
@@ -138,9 +150,13 @@ fn point_curve_intersections<S: Scalar>(
     // or if the curve has a cusp in one of the corners of its convex hull and pt is
     // diagonally just outside the hull.  This is a rare case (could we even ignore it?).
     #[inline]
-    fn maybe_add<S: Scalar>(t: S, pt: &Point<S>, curve: &CubicBezierSegment<S>, epsilon: S,
-                            result: &mut ArrayVec<[S; 9]>) -> bool
-    {
+    fn maybe_add<S: Scalar>(
+        t: S,
+        pt: &Point<S>,
+        curve: &CubicBezierSegment<S>,
+        epsilon: S,
+        result: &mut ArrayVec<S, 9>,
+    ) -> bool {
         if (curve.sample(t) - *pt).square_length() < epsilon {
             result.push(t);
             return true;
@@ -160,7 +176,7 @@ fn line_curve_intersections<S: Scalar>(
     line_as_curve: &CubicBezierSegment<S>,
     curve: &CubicBezierSegment<S>,
     flip: bool,
-) -> ArrayVec<[(S, S); 9]> {
+) -> ArrayVec<(S, S), 9> {
     let mut result = ArrayVec::new();
     let baseline = line_as_curve.baseline();
     let curve_intersections = curve.line_intersections_t(&baseline.to_line());
@@ -186,10 +202,13 @@ fn line_curve_intersections<S: Scalar>(
 fn line_line_intersections<S: Scalar>(
     curve1: &CubicBezierSegment<S>,
     curve2: &CubicBezierSegment<S>,
-) -> ArrayVec<[(S, S); 9]> {
+) -> ArrayVec<(S, S), 9> {
     let mut result = ArrayVec::new();
 
-    let intersection = curve1.baseline().to_line().intersection(&curve2.baseline().to_line());
+    let intersection = curve1
+        .baseline()
+        .to_line()
+        .intersection(&curve2.baseline().to_line());
     if intersection.is_none() {
         return result;
     }
@@ -200,7 +219,7 @@ fn line_line_intersections<S: Scalar>(
     fn parameters_for_line_point<S: Scalar>(
         curve: &CubicBezierSegment<S>,
         pt: &Point<S>,
-    ) -> ArrayVec<[S; 3]> {
+    ) -> ArrayVec<S, 3> {
         let line_is_mostly_vertical =
             S::abs(curve.from.y - curve.to.y) >= S::abs(curve.from.x - curve.to.x);
         if line_is_mostly_vertical {
@@ -210,13 +229,13 @@ fn line_line_intersections<S: Scalar>(
         }
     }
 
-    let line1_params = parameters_for_line_point(&curve1, &intersection);
-    if line1_params.len() == 0 {
+    let line1_params = parameters_for_line_point(curve1, &intersection);
+    if line1_params.is_empty() {
         return result;
     }
 
-    let line2_params = parameters_for_line_point(&curve2, &intersection);
-    if line2_params.len() == 0 {
+    let line2_params = parameters_for_line_point(curve2, &intersection);
+    if line2_params.is_empty() {
         return result;
     }
 
@@ -236,33 +255,44 @@ fn line_line_intersections<S: Scalar>(
 // the curves intersects the fat line of the other curve at each stage.
 //
 // curve1 and curve2 at each stage are sub-bézier curves of the original curves; flip tells us
-// whether curve1 at a given stage is a subcurve of the original curve1 or the original curve2;
+// whether curve1 at a given stage is a sub-curve of the original curve1 or the original curve2;
 // similarly for curve2.  domain1 and domain2 shrink (or stay the same) at each stage and describe
 // which subdomain of an original curve the current curve1 and curve2 correspond to. (The domains of
 // curve1 and curve2 are 0..1 at every stage.)
+#[allow(clippy::too_many_arguments)]
 fn add_curve_intersections<S: Scalar>(
     curve1: &CubicBezierSegment<S>,
     curve2: &CubicBezierSegment<S>,
     domain1: &Range<S>,
     domain2: &Range<S>,
-    intersections: &mut ArrayVec<[(S, S); 9]>,
+    intersections: &mut ArrayVec<(S, S), 9>,
     flip: bool,
     mut recursion_count: u32,
     mut call_count: u32,
     orig_curve1: &CubicBezierSegment<S>,
     orig_curve2: &CubicBezierSegment<S>,
-) -> u32  {
+) -> u32 {
     call_count += 1;
     recursion_count += 1;
     if call_count >= 4096 || recursion_count >= 60 {
         return call_count;
     }
 
-    let epsilon = if inputs_are_f32::<S>() { S::value(5e-6) } else { S::value(1e-9) };
+    let epsilon = if inputs_are_f32::<S>() {
+        S::value(5e-6)
+    } else {
+        S::value(1e-9)
+    };
 
     if domain2.start == domain2.end || curve2.is_a_point(S::ZERO) {
         add_point_curve_intersection(
-            &curve2, /* point is curve1 */ false, curve1, domain2, domain1, intersections, flip,
+            curve2,
+            /* point is curve1 */ false,
+            curve1,
+            domain2,
+            domain1,
+            intersections,
+            flip,
         );
         return call_count;
     } else if curve2.from == curve2.to {
@@ -271,21 +301,35 @@ fn add_curve_intersections<S: Scalar>(
         let new_2_curves = orig_curve2.split_range(domain2.clone()).split(S::HALF);
         let domain2_mid = (domain2.start + domain2.end) * S::HALF;
         call_count = add_curve_intersections(
-            curve1, &new_2_curves.0, domain1, &(domain2.start..domain2_mid),
-            intersections, flip, recursion_count, call_count,
-            orig_curve1, orig_curve2,
+            curve1,
+            &new_2_curves.0,
+            domain1,
+            &(domain2.start..domain2_mid),
+            intersections,
+            flip,
+            recursion_count,
+            call_count,
+            orig_curve1,
+            orig_curve2,
         );
         call_count = add_curve_intersections(
-            curve1, &new_2_curves.1, domain1, &(domain2_mid..domain2.end),
-            intersections, flip, recursion_count, call_count,
-            orig_curve1, orig_curve2,
+            curve1,
+            &new_2_curves.1,
+            domain1,
+            &(domain2_mid..domain2.end),
+            intersections,
+            flip,
+            recursion_count,
+            call_count,
+            orig_curve1,
+            orig_curve2,
         );
         return call_count;
     }
 
     // (Don't call this before checking for point curves: points are inexact and can lead to false
     // negatives here.)
-    if !rectangles_overlap(&curve1.fast_bounding_rect(), &curve2.fast_bounding_rect()) {
+    if !rectangles_overlap(&curve1.fast_bounding_box(), &curve2.fast_bounding_box()) {
         return call_count;
     }
 
@@ -297,9 +341,13 @@ fn add_curve_intersections<S: Scalar>(
     // t_min_clip and t_max_clip are (0, 1)-based, so project them back to get the new restricted
     // range:
     let new_domain1 =
-        &(domain_value_at_t(&domain1, t_min_clip)..domain_value_at_t(&domain1, t_max_clip));
+        &(domain_value_at_t(domain1, t_min_clip)..domain_value_at_t(domain1, t_max_clip));
 
-    if S::max(domain2.end - domain2.start, new_domain1.end - new_domain1.start) < epsilon {
+    if S::max(
+        domain2.end - domain2.start,
+        new_domain1.end - new_domain1.start,
+    ) < epsilon
+    {
         let t1 = (new_domain1.start + new_domain1.end) * S::HALF;
         let t2 = (domain2.start + domain2.end) * S::HALF;
         if inputs_are_f32::<S>() {
@@ -325,8 +373,13 @@ fn add_curve_intersections<S: Scalar>(
     // is a point (but then curve1 will be very small).)
     if new_domain1.start == new_domain1.end || curve1.is_a_point(S::ZERO) {
         add_point_curve_intersection(
-            &curve1, /* point is curve1 */ true, curve2, new_domain1, domain2,
-            intersections, flip,
+            curve1,
+            /* point is curve1 */ true,
+            curve2,
+            new_domain1,
+            domain2,
+            intersections,
+            flip,
         );
         return call_count;
     }
@@ -338,43 +391,85 @@ fn add_curve_intersections<S: Scalar>(
             let new_1_curves = curve1.split(S::HALF);
             let new_domain1_mid = (new_domain1.start + new_domain1.end) * S::HALF;
             call_count = add_curve_intersections(
-                curve2, &new_1_curves.0, domain2, &(new_domain1.start..new_domain1_mid),
-                intersections, !flip, recursion_count, call_count,
-                orig_curve2, orig_curve1,
+                curve2,
+                &new_1_curves.0,
+                domain2,
+                &(new_domain1.start..new_domain1_mid),
+                intersections,
+                !flip,
+                recursion_count,
+                call_count,
+                orig_curve2,
+                orig_curve1,
             );
             call_count = add_curve_intersections(
-                curve2, &new_1_curves.1, domain2, &(new_domain1_mid..new_domain1.end),
-                intersections, !flip, recursion_count, call_count,
-                orig_curve2, orig_curve1,
+                curve2,
+                &new_1_curves.1,
+                domain2,
+                &(new_domain1_mid..new_domain1.end),
+                intersections,
+                !flip,
+                recursion_count,
+                call_count,
+                orig_curve2,
+                orig_curve1,
             );
         } else {
             let new_2_curves = orig_curve2.split_range(domain2.clone()).split(S::HALF);
             let domain2_mid = (domain2.start + domain2.end) * S::HALF;
             call_count = add_curve_intersections(
-                &new_2_curves.0, curve1, &(domain2.start..domain2_mid), new_domain1,
-                intersections, !flip, recursion_count, call_count,
-                orig_curve2, orig_curve1,
+                &new_2_curves.0,
+                curve1,
+                &(domain2.start..domain2_mid),
+                new_domain1,
+                intersections,
+                !flip,
+                recursion_count,
+                call_count,
+                orig_curve2,
+                orig_curve1,
             );
             call_count = add_curve_intersections(
-                &new_2_curves.1, curve1, &(domain2_mid..domain2.end), new_domain1,
-                intersections, !flip, recursion_count, call_count,
-                orig_curve2, orig_curve1,
+                &new_2_curves.1,
+                curve1,
+                &(domain2_mid..domain2.end),
+                new_domain1,
+                intersections,
+                !flip,
+                recursion_count,
+                call_count,
+                orig_curve2,
+                orig_curve1,
             );
         }
     } else {
         // Iterate.
         if domain2.end - domain2.start >= epsilon {
             call_count = add_curve_intersections(
-                curve2, curve1, domain2, new_domain1,
-                intersections, !flip, recursion_count, call_count,
-                orig_curve2, orig_curve1,
+                curve2,
+                curve1,
+                domain2,
+                new_domain1,
+                intersections,
+                !flip,
+                recursion_count,
+                call_count,
+                orig_curve2,
+                orig_curve1,
             );
         } else {
             // The interval on curve2 is already tight enough, so just continue iterating on curve1.
             call_count = add_curve_intersections(
-                curve1, curve2, new_domain1, domain2,
-                intersections, flip, recursion_count, call_count,
-                orig_curve1, orig_curve2,
+                curve1,
+                curve2,
+                new_domain1,
+                domain2,
+                intersections,
+                flip,
+                recursion_count,
+                call_count,
+                orig_curve1,
+                orig_curve2,
             );
         }
     }
@@ -388,7 +483,7 @@ fn add_point_curve_intersection<S: Scalar>(
     curve: &CubicBezierSegment<S>,
     pt_domain: &Range<S>,
     curve_domain: &Range<S>,
-    intersections: &mut ArrayVec<[(S, S); 9]>,
+    intersections: &mut ArrayVec<(S, S), 9>,
     flip: bool,
 ) {
     let pt = pt_curve.from;
@@ -435,24 +530,25 @@ fn add_point_curve_intersection<S: Scalar>(
     }
 }
 
+// TODO: replace with Scalar::epsilon_for?
 // If we're comparing distances between samples of curves, our epsilon should depend on how big the
 // points we're comparing are. This function returns an epsilon appropriate for the size of pt.
 fn epsilon_for_point<S: Scalar>(pt: &Point<S>) -> S {
     let max = S::max(S::abs(pt.x), S::abs(pt.y));
     let epsilon = if inputs_are_f32::<S>() {
         match max.to_i32().unwrap() {
-            0 ..= 9 => S::value(0.001),
-            10 ..= 99 => S::value(0.01),
-            100 ..= 999 => S::value(0.1),
-            1_000 ..= 9_999 => S::value(0.25),
-            10_000 ..= 999_999 => S::HALF,
+            0..=9 => S::value(0.001),
+            10..=99 => S::value(0.01),
+            100..=999 => S::value(0.1),
+            1_000..=9_999 => S::value(0.25),
+            10_000..=999_999 => S::HALF,
             _ => S::ONE,
         }
     } else {
         match max.to_i64().unwrap() {
-            0 ..= 99_999 => S::EPSILON,
-            100_000 ..= 99_999_999 => S::value(1e-5),
-            100_000_000 ..= 9_999_999_999 => S::value(1e-3),
+            0..=99_999 => S::EPSILON,
+            100_000..=99_999_999 => S::value(1e-5),
+            100_000_000..=9_999_999_999 => S::value(1e-3),
             _ => S::value(1e-1),
         }
     };
@@ -466,11 +562,15 @@ fn add_intersection<S: Scalar>(
     t2: S,
     orig_curve2: &CubicBezierSegment<S>,
     flip: bool,
-    intersections: &mut ArrayVec<[(S, S); 9]>,
+    intersections: &mut ArrayVec<(S, S), 9>,
 ) {
     let (t1, t2) = if flip { (t2, t1) } else { (t1, t2) };
     // (This should probably depend in some way on how large our input coefficients are.)
-    let epsilon = if inputs_are_f32::<S>() { S::value(1e-3) } else { S::EPSILON };
+    let epsilon = if inputs_are_f32::<S>() {
+        S::value(1e-3)
+    } else {
+        S::EPSILON
+    };
     // Discard endpoint/endpoint intersections.
     let t1_is_an_endpoint = t1 < epsilon || t1 > S::ONE - epsilon;
     let t2_is_an_endpoint = t2 < epsilon || t2 > S::ONE - epsilon;
@@ -520,7 +620,9 @@ fn restrict_curve_to_fat_line<S: Scalar>(
     let d_2 = baseline2.signed_distance_to_point(&curve1.ctrl2);
     let d_3 = baseline2.signed_distance_to_point(&curve1.to);
 
-    let (mut top, mut bottom) = convex_hull_of_distance_curve(d_0, d_1, d_2, d_3);
+    let mut top = ArrayVec::new();
+    let mut bottom = ArrayVec::new();
+    convex_hull_of_distance_curve(d_0, d_1, d_2, d_3, &mut top, &mut bottom);
     let (d_min, d_max) = curve2.fat_line_min_max();
 
     clip_convex_hull_to_fat_line(&mut top, &mut bottom, d_min, d_max)
@@ -535,7 +637,12 @@ fn convex_hull_of_distance_curve<S: Scalar>(
     d1: S,
     d2: S,
     d3: S,
-) -> (Vec<Point<S>>, Vec<Point<S>>) {
+    top: &mut ArrayVec<Point<S>, 4>,
+    bottom: &mut ArrayVec<Point<S>, 4>,
+) {
+    debug_assert!(top.is_empty());
+    debug_assert!(bottom.is_empty());
+
     let p0 = point(S::ZERO, d0);
     let p1 = point(S::ONE / S::THREE, d1);
     let p2 = point(S::TWO / S::THREE, d2);
@@ -545,9 +652,10 @@ fn convex_hull_of_distance_curve<S: Scalar>(
     let dist2 = d2 - (d0 + S::TWO * d3) / S::THREE;
 
     // Compute the hull assuming p1 is on top - we'll switch later if needed.
-    let mut hull = if dist1 * dist2 < S::ZERO {
+    if dist1 * dist2 < S::ZERO {
         // p1 and p2 lie on opposite sides of [p0, p3], so the hull is a quadrilateral:
-        (vec![p0, p1, p3], vec![p0, p2, p3])
+        let _ = top.try_extend_from_slice(&[p0, p1, p3]);
+        let _ = bottom.try_extend_from_slice(&[p0, p2, p3]);
     } else {
         // p1 and p2 lie on the same side of [p0, p3]. The hull can be a triangle or a
         // quadrilateral, and [p0, p3] is part of the hull. The hull is a triangle if the vertical
@@ -556,68 +664,65 @@ fn convex_hull_of_distance_curve<S: Scalar>(
         let dist1 = S::abs(dist1);
         let dist2 = S::abs(dist2);
         if dist1 >= S::TWO * dist2 {
-            (vec![p0, p1, p3], vec![p0, p3])
+            let _ = top.try_extend_from_slice(&[p0, p1, p3]);
+            let _ = bottom.try_extend_from_slice(&[p0, p3]);
         } else if dist2 >= S::TWO * dist1 {
-            (vec![p0, p2, p3], vec![p0, p3])
+            let _ = top.try_extend_from_slice(&[p0, p2, p3]);
+            let _ = bottom.try_extend_from_slice(&[p0, p3]);
         } else {
-            (vec![p0, p1, p2, p3], vec![p0, p3])
+            let _ = top.try_extend_from_slice(&[p0, p1, p2, p3]);
+            let _ = bottom.try_extend_from_slice(&[p0, p3]);
         }
-    };
+    }
 
     // Flip the hull if needed:
     if dist1 < S::ZERO || (dist1 == S::ZERO && dist2 < S::ZERO) {
-        hull = (hull.1, hull.0);
+        mem::swap(top, bottom);
     }
-
-    hull
 }
 
 // Returns the min and max values at which the convex hull enters the fat line min/max offset lines.
 fn clip_convex_hull_to_fat_line<S: Scalar>(
-    hull_top: &mut Vec<Point<S>>,
-    hull_bottom: &mut Vec<Point<S>>,
+    hull_top: &mut [Point<S>],
+    hull_bottom: &mut [Point<S>],
     d_min: S,
     d_max: S,
 ) -> Option<(S, S)> {
     // Walk from the left corner of the convex hull until we enter the fat line limits:
-    let t_clip_min = walk_convex_hull_start_to_fat_line(&hull_top, &hull_bottom, d_min, d_max);
-    if let None = t_clip_min {
-        return None;
-    }
+    let t_clip_min = walk_convex_hull_start_to_fat_line(hull_top, hull_bottom, d_min, d_max)?;
+
     // Now walk from the right corner of the convex hull until we enter the fat line limits - to
     // walk right to left we just reverse the order of the hull vertices, so that hull_top and
     // hull_bottom start at the right corner now:
     hull_top.reverse();
     hull_bottom.reverse();
-    let t_clip_max = walk_convex_hull_start_to_fat_line(&hull_top, &hull_bottom, d_min, d_max);
-    if let None = t_clip_max {
-        return None;
-    }
-    Some((t_clip_min.unwrap(), t_clip_max.unwrap()))
+    let t_clip_max = walk_convex_hull_start_to_fat_line(hull_top, hull_bottom, d_min, d_max)?;
+
+    Some((t_clip_min, t_clip_max))
 }
 
 // Walk the edges of the convex hull until you hit a fat line offset value, starting from the
 // (first vertex in hull_top_vertices == first vertex in hull_bottom_vertices).
 fn walk_convex_hull_start_to_fat_line<S: Scalar>(
-    hull_top_vertices: &Vec<Point<S>>,
-    hull_bottom_vertices: &Vec<Point<S>>,
+    hull_top_vertices: &[Point<S>],
+    hull_bottom_vertices: &[Point<S>],
     d_min: S,
     d_max: S,
 ) -> Option<S> {
     let start_corner = hull_top_vertices[0];
 
     if start_corner.y < d_min {
-        return walk_convex_hull_edges_to_fat_line(hull_top_vertices, true, d_min);
+        walk_convex_hull_edges_to_fat_line(hull_top_vertices, true, d_min)
     } else if start_corner.y > d_max {
-        return walk_convex_hull_edges_to_fat_line(hull_bottom_vertices, false, d_max);
+        walk_convex_hull_edges_to_fat_line(hull_bottom_vertices, false, d_max)
     } else {
-        return Some(start_corner.x);
+        Some(start_corner.x)
     }
 }
 
 // Do the actual walking, starting from the first vertex of hull_vertices.
 fn walk_convex_hull_edges_to_fat_line<S: Scalar>(
-    hull_vertices: &Vec<Point<S>>,
+    hull_vertices: &[Point<S>],
     vertices_are_for_top: bool,
     threshold: S,
 ) -> Option<S> {
@@ -626,11 +731,11 @@ fn walk_convex_hull_edges_to_fat_line<S: Scalar>(
         let q = hull_vertices[i + 1];
         if (vertices_are_for_top && q.y >= threshold) || (!vertices_are_for_top && q.y <= threshold)
         {
-            if q.y == threshold {
-                return Some(q.x);
+            return if q.y == threshold {
+                Some(q.x)
             } else {
-                return Some(p.x + (threshold - p.y) * (q.x - p.x) / (q.y - p.y));
-            }
+                Some(p.x + (threshold - p.y) * (q.x - p.x) / (q.y - p.y))
+            };
         }
     }
     // All points of the hull are outside the threshold:
@@ -649,12 +754,9 @@ fn domain_value_at_t<S: Scalar>(domain: &Range<S>, t: S) -> S {
 }
 
 #[inline]
-// Rect.intersects doesn't count edge/corner intersections, this version does.
-fn rectangles_overlap<S: Scalar>(r1: &Rect<S>, r2: &Rect<S>) -> bool {
-    r1.origin.x <= r2.origin.x + r2.size.width
-        && r2.origin.x <= r1.origin.x + r1.size.width
-        && r1.origin.y <= r2.origin.y + r2.size.height
-        && r2.origin.y <= r1.origin.y + r1.size.height
+// Box2D::intersects doesn't count edge/corner intersections, this version does.
+fn rectangles_overlap<S: Scalar>(r1: &Box2D<S>, r2: &Box2D<S>) -> bool {
+    r1.min.x <= r2.max.x && r2.min.x <= r1.max.x && r1.min.y <= r2.max.y && r2.min.y <= r1.max.y
 }
 
 #[cfg(test)]
@@ -673,7 +775,7 @@ fn do_test_once<S: Scalar>(
     curve2: &CubicBezierSegment<S>,
     intersection_count: i32,
 ) {
-    let intersections = cubic_bezier_intersections_t(&curve1, &curve2);
+    let intersections = cubic_bezier_intersections_t(curve1, curve2);
     for intersection in &intersections {
         let p1 = curve1.sample(intersection.0);
         let p2 = curve2.sample(intersection.1);
@@ -798,7 +900,7 @@ fn test_cubic_control_point_touching() {
     // After splitting the curve2 loop in half, curve1.ctrl1 (and only that
     // point) touches the curve2 fat line - make sure we don't accept that as an
     // intersection. [We're really only interested in the right half of the loop - the rest of the
-    // loop is there just to get past an initial fast_bounding_rect check.]
+    // loop is there just to get past an initial fast_bounding_box check.]
     do_test(
         &CubicBezierSegment {
             from: point(-1.0, 0.0),
@@ -1126,46 +1228,42 @@ fn test_cubic_interior_endpoint() {
 fn test_cubic_point_curve_intersections() {
     let epsilon = 1e-5;
     {
-        let curve1 =
-            CubicBezierSegment {
-                from: point(0.0, 0.0),
-                ctrl1: point(0.0, 1.0),
-                ctrl2: point(0.0, 1.0),
-                to: point(1.0, 1.0),
-            };
+        let curve1 = CubicBezierSegment {
+            from: point(0.0, 0.0),
+            ctrl1: point(0.0, 1.0),
+            ctrl2: point(0.0, 1.0),
+            to: point(1.0, 1.0),
+        };
         let sample_t = 0.123456789;
         let pt = curve1.sample(sample_t);
-        let curve2 =
-            CubicBezierSegment {
-                from: pt,
-                ctrl1: pt,
-                ctrl2: pt,
-                to: pt,
-            };
+        let curve2 = CubicBezierSegment {
+            from: pt,
+            ctrl1: pt,
+            ctrl2: pt,
+            to: pt,
+        };
         let intersections = cubic_bezier_intersections_t(&curve1, &curve2);
         assert_eq!(intersections.len(), 1);
         let intersection_t = intersections[0].0;
         assert!(f64::abs(intersection_t - sample_t) < epsilon);
     }
     {
-        let curve1 =
-            CubicBezierSegment {
-                from: point(-10.0, -13.636363636363636),
-                ctrl1: point(15.0, 11.363636363636363),
-                ctrl2: point(-15.0, 11.363636363636363),
-                to: point(10.0, -13.636363636363636),
-            };
+        let curve1 = CubicBezierSegment {
+            from: point(-10.0, -13.636363636363636),
+            ctrl1: point(15.0, 11.363636363636363),
+            ctrl2: point(-15.0, 11.363636363636363),
+            to: point(10.0, -13.636363636363636),
+        };
         // curve1 has a self intersection at the following parameter values:
         let parameter1 = 0.7611164839335472;
         let parameter2 = 0.23888351606645375;
         let pt = curve1.sample(parameter1);
-        let curve2 =
-            CubicBezierSegment {
-                from: pt,
-                ctrl1: pt,
-                ctrl2: pt,
-                to: pt,
-            };
+        let curve2 = CubicBezierSegment {
+            from: pt,
+            ctrl1: pt,
+            ctrl2: pt,
+            to: pt,
+        };
         let intersections = cubic_bezier_intersections_t(&curve1, &curve2);
         assert_eq!(intersections.len(), 2);
         let intersection_t1 = intersections[0].0;
@@ -1175,25 +1273,23 @@ fn test_cubic_point_curve_intersections() {
     }
     {
         let epsilon = epsilon as f32;
-        let curve1 =
-            CubicBezierSegment {
-                from: point(0.0f32, 0.0),
-                ctrl1: point(50.0, 50.0),
-                ctrl2: point(-50.0, -50.0),
-                to: point(10.0, 10.0),
-            };
+        let curve1 = CubicBezierSegment {
+            from: point(0.0f32, 0.0),
+            ctrl1: point(50.0, 50.0),
+            ctrl2: point(-50.0, -50.0),
+            to: point(10.0, 10.0),
+        };
         // curve1 is a line that passes through (5.0, 5.0) three times:
         let parameter1 = 0.96984464;
         let parameter2 = 0.037427425;
         let parameter3 = 0.44434106;
         let pt = curve1.sample(parameter1);
-        let curve2 =
-            CubicBezierSegment {
-                from: pt,
-                ctrl1: pt,
-                ctrl2: pt,
-                to: pt,
-            };
+        let curve2 = CubicBezierSegment {
+            from: pt,
+            ctrl1: pt,
+            ctrl2: pt,
+            to: pt,
+        };
         let intersections = cubic_bezier_intersections_t(&curve1, &curve2);
         assert_eq!(intersections.len(), 3);
         let intersection_t1 = intersections[0].0;
