@@ -1,12 +1,11 @@
 use crate::scalar::Scalar;
-use crate::generic_math::{Point, point, Vector, vector, Rect, Size};
-use crate::segment::{Segment, FlatteningStep, BoundingRect};
-use crate::monotonic::MonotonicSegment;
-use crate::utils::min_max;
+use crate::segment::{BoundingBox, Segment};
 use crate::traits::Transformation;
-use std::mem::swap;
+use crate::utils::min_max;
+use crate::{point, vector, Box2D, Point, Vector};
+use core::mem::swap;
 
-use std::ops::Range;
+use core::ops::Range;
 
 /// A linear segment.
 #[derive(Copy, Clone, Debug, PartialEq)]
@@ -36,10 +35,14 @@ impl<S: Scalar> LineSegment<S> {
     }
 
     #[inline]
-    pub fn from(&self) -> Point<S> { self.from }
+    pub fn from(&self) -> Point<S> {
+        self.from
+    }
 
     #[inline]
-    pub fn to(&self) -> Point<S> { self.to }
+    pub fn to(&self) -> Point<S> {
+        self.to
+    }
 
     pub fn solve_t_for_x(&self, x: S) -> S {
         let dx = self.to.x - self.from.x;
@@ -53,7 +56,7 @@ impl<S: Scalar> LineSegment<S> {
     pub fn solve_t_for_y(&self, y: S) -> S {
         let dy = self.to.y - self.from.y;
         if dy == S::ZERO {
-            return S::ZERO
+            return S::ZERO;
         }
 
         (y - self.from.y) / dy
@@ -71,7 +74,10 @@ impl<S: Scalar> LineSegment<S> {
     /// points are swapped.
     #[inline]
     pub fn flip(&self) -> Self {
-        LineSegment { from: self.to, to: self.from }
+        LineSegment {
+            from: self.to,
+            to: self.from,
+        }
     }
 
     /// Return the sub-segment inside a given range of t.
@@ -88,37 +94,51 @@ impl<S: Scalar> LineSegment<S> {
     #[inline]
     pub fn split(&self, t: S) -> (Self, Self) {
         let split_point = self.sample(t);
-        return (
-            LineSegment { from: self.from, to: split_point },
-            LineSegment { from: split_point, to: self.to },
-        );
+
+        (
+            LineSegment {
+                from: self.from,
+                to: split_point,
+            },
+            LineSegment {
+                from: split_point,
+                to: self.to,
+            },
+        )
     }
 
     /// Return the segment before the split point.
     #[inline]
     pub fn before_split(&self, t: S) -> Self {
-        LineSegment { from: self.from, to: self.sample(t) }
+        LineSegment {
+            from: self.from,
+            to: self.sample(t),
+        }
     }
 
     /// Return the segment after the split point.
     #[inline]
     pub fn after_split(&self, t: S) -> Self {
-        LineSegment { from: self.sample(t), to: self.to }
+        LineSegment {
+            from: self.sample(t),
+            to: self.to,
+        }
     }
 
     pub fn split_at_x(&self, x: S) -> (Self, Self) {
         self.split(self.solve_t_for_x(x))
     }
 
-    /// Return the minimum bounding rectangle
+    /// Return the smallest rectangle containing this segment.
     #[inline]
-    pub fn bounding_rect(&self) -> Rect<S> {
+    pub fn bounding_box(&self) -> Box2D<S> {
         let (min_x, max_x) = self.bounding_range_x();
         let (min_y, max_y) = self.bounding_range_y();
 
-        let width  = max_x - min_x;
-        let height = max_y - min_y;
-        Rect::new(Point::new(min_x, min_y), Size::new(width, height))
+        Box2D {
+            min: point(min_x, min_y),
+            max: point(max_x, max_y),
+        }
     }
 
     #[inline]
@@ -152,6 +172,12 @@ impl<S: Scalar> LineSegment<S> {
         self.to_vector().length()
     }
 
+    /// Computes the squared length of this segment.
+    #[inline]
+    pub fn square_length(&self) -> S {
+        self.to_vector().square_length()
+    }
+
     /// Changes the segment's length, moving destination point.
     pub fn set_length(&mut self, new_length: S) {
         let v = self.to_vector();
@@ -159,6 +185,11 @@ impl<S: Scalar> LineSegment<S> {
         self.to = self.from + v * (new_length / old_length);
     }
 
+    /// Computes third mid-point of this segment.
+    pub fn mid_point(&mut self) -> Point<S> {
+        (self.from + self.to.to_vector()) / S::TWO
+    }
+
     #[inline]
     pub fn translate(&mut self, by: Vector<S>) -> Self {
         LineSegment {
@@ -181,17 +212,13 @@ impl<S: Scalar> LineSegment<S> {
     /// The result is provided in the form of the `t` parameter of each
     /// segment. To get the intersection point, sample one of the segments
     /// at the corresponding value.
+    #[allow(clippy::suspicious_operation_groupings)]
     pub fn intersection_t(&self, other: &Self) -> Option<(S, S)> {
-        let (min1, max1) = self.bounding_range_x();
-        let (min2, max2) = other.bounding_range_x();
-        if min1 > max2 || max1 < min2 {
-            return None;
-        }
-
         if self.to == other.to
             || self.from == other.from
             || self.from == other.to
-            || self.to == other.from {
+            || self.to == other.from
+        {
             return None;
         }
 
@@ -220,10 +247,7 @@ impl<S: Scalar> LineSegment<S> {
             return None;
         }
 
-        Some((
-            t / abs_v1_cross_v2,
-            u / abs_v1_cross_v2,
-        ))
+        Some((t / abs_v1_cross_v2, u / abs_v1_cross_v2))
     }
 
     #[inline]
@@ -260,6 +284,7 @@ impl<S: Scalar> LineSegment<S> {
         self.line_intersection_t(line).map(|t| self.sample(t))
     }
 
+    // TODO: Consider only intersecting in the [0, 1[ range instead of [0, 1]
     pub fn horizontal_line_intersection_t(&self, y: S) -> Option<S> {
         Self::axis_aligned_intersection_1d(self.from.y, self.to.y, y)
     }
@@ -270,7 +295,8 @@ impl<S: Scalar> LineSegment<S> {
 
     #[inline]
     pub fn horizontal_line_intersection(&self, y: S) -> Option<Point<S>> {
-        self.horizontal_line_intersection_t(y).map(|t| self.sample(t))
+        self.horizontal_line_intersection_t(y)
+            .map(|t| self.sample(t))
     }
 
     #[inline]
@@ -279,9 +305,10 @@ impl<S: Scalar> LineSegment<S> {
     }
 
     fn axis_aligned_intersection_1d(mut a: S, mut b: S, v: S) -> Option<S> {
+        // TODO: is it really useful to swap?
         let swap = a > b;
         if swap {
-            std::mem::swap(&mut a, &mut b);
+            core::mem::swap(&mut a, &mut b);
         }
 
         let d = b - a;
@@ -365,45 +392,177 @@ impl<S: Scalar> LineSegment<S> {
 
         c >= a && c <= b && d >= a && d <= b
     }
+
+    /// Horizontally clip this segment against a range of the x axis.
+    pub fn clipped_x(&self, clip: Range<S>) -> Option<Self> {
+        if (self.from.x < clip.start && self.to.x < clip.start)
+            || (self.from.x > clip.end && self.to.x > clip.end)
+        {
+            return None;
+        }
+
+        let mut flipped = false;
+        let mut result = *self;
+
+        if result.from.x > result.to.x {
+            flipped = true;
+            result = result.flip();
+        }
+
+        if result.from.x >= clip.start && result.to.x <= clip.end {
+            return Some(*self);
+        }
+
+        if result.from.x < clip.start {
+            let t = result
+                .vertical_line_intersection_t(clip.start)
+                .unwrap_or(S::ZERO);
+            result.from.x = clip.start;
+            result.from.y = result.y(t);
+        }
+
+        if result.to.x > clip.end {
+            let t = result
+                .vertical_line_intersection_t(clip.end)
+                .unwrap_or(S::ZERO);
+            result.to.x = clip.end;
+            result.to.y = result.y(t);
+        }
+
+        if flipped {
+            result = result.flip();
+        }
+
+        Some(result)
+    }
+
+    /// Vertically clip this segment against a range of the y axis.
+    pub fn clipped_y(&self, clip: Range<S>) -> Option<Self> {
+        fn transpose<S: Copy>(r: &LineSegment<S>) -> LineSegment<S> {
+            LineSegment {
+                from: r.from.yx(),
+                to: r.to.yx(),
+            }
+        }
+
+        Some(transpose(&transpose(self).clipped_x(clip)?))
+    }
+
+    /// Clip this segment against a rectangle.
+    pub fn clipped(&self, clip: &Box2D<S>) -> Option<Self> {
+        self.clipped_x(clip.x_range())?.clipped_y(clip.y_range())
+    }
+
+    /// Computes the distance between this segment and a point.
+    #[inline]
+    pub fn distance_to_point(&self, p: Point<S>) -> S {
+        self.square_distance_to_point(p).sqrt()
+    }
+
+    /// Computes the squared distance between this segment and a point.
+    ///
+    /// Can be useful to save a square root and a division when comparing against
+    /// a distance that can be squared.
+    #[inline]
+    pub fn square_distance_to_point(&self, p: Point<S>) -> S {
+        (self.closest_point(p) - p).square_length()
+    }
+
+    /// Computes the closest point on this segment to `p`.
+    #[inline]
+    pub fn closest_point(&self, p: Point<S>) -> Point<S> {
+        let v1 = self.to - self.from;
+        let v2 = p - self.from;
+        let t = S::min(S::max(v2.dot(v1) / v1.dot(v1), S::ZERO), S::ONE);
+
+        self.from + v1 * t
+    }
+
+    #[inline]
+    pub fn to_f32(&self) -> LineSegment<f32> {
+        LineSegment {
+            from: self.from.to_f32(),
+            to: self.to.to_f32(),
+        }
+    }
+
+    #[inline]
+    pub fn to_f64(&self) -> LineSegment<f64> {
+        LineSegment {
+            from: self.from.to_f64(),
+            to: self.to.to_f64(),
+        }
+    }
 }
 
 impl<S: Scalar> Segment for LineSegment<S> {
     type Scalar = S;
-    fn from(&self) -> Point<S> { self.from }
-    fn to(&self) -> Point<S> { self.to }
-    fn sample(&self, t: S) -> Point<S> { self.sample(t) }
-    fn x(&self, t: S) -> S { self.x(t) }
-    fn y(&self, t: S) -> S { self.y(t) }
-    fn derivative(&self, _t: S) -> Vector<S> { self.to_vector() }
-    fn dx(&self, _t: S) -> S { self.to.x - self.from.x }
-    fn dy(&self, _t: S) -> S { self.to.y - self.from.y }
-    fn split_range(&self, t_range: Range<S>) -> Self { self.split_range(t_range) }
-    fn split(&self, t: S) -> (Self, Self) { self.split(t) }
-    fn before_split(&self, t: S) -> Self { self.before_split(t) }
-    fn after_split(&self, t: S) -> Self { self.after_split(t) }
-    fn flip(&self) -> Self { self.flip() }
-    fn approximate_length(&self, _tolerance: S) -> S { self.length() }
-}
+    fn from(&self) -> Point<S> {
+        self.from
+    }
+    fn to(&self) -> Point<S> {
+        self.to
+    }
+    fn sample(&self, t: S) -> Point<S> {
+        self.sample(t)
+    }
+    fn x(&self, t: S) -> S {
+        self.x(t)
+    }
+    fn y(&self, t: S) -> S {
+        self.y(t)
+    }
+    fn derivative(&self, _t: S) -> Vector<S> {
+        self.to_vector()
+    }
+    fn dx(&self, _t: S) -> S {
+        self.to.x - self.from.x
+    }
+    fn dy(&self, _t: S) -> S {
+        self.to.y - self.from.y
+    }
+    fn split(&self, t: S) -> (Self, Self) {
+        self.split(t)
+    }
+    fn before_split(&self, t: S) -> Self {
+        self.before_split(t)
+    }
+    fn after_split(&self, t: S) -> Self {
+        self.after_split(t)
+    }
+    fn split_range(&self, t_range: Range<S>) -> Self {
+        self.split_range(t_range)
+    }
+    fn flip(&self) -> Self {
+        self.flip()
+    }
+    fn approximate_length(&self, _tolerance: S) -> S {
+        self.length()
+    }
 
-impl<S: Scalar> BoundingRect for LineSegment<S> {
-    type Scalar = S;
-    fn bounding_rect(&self) -> Rect<S> { self.bounding_rect() }
-    fn fast_bounding_rect(&self) -> Rect<S> { self.bounding_rect() }
-    fn bounding_range_x(&self) -> (S, S) { self.bounding_range_x() }
-    fn bounding_range_y(&self) -> (S, S) { self.bounding_range_y() }
-    fn fast_bounding_range_x(&self) -> (S, S) { self.bounding_range_x() }
-    fn fast_bounding_range_y(&self) -> (S, S) { self.bounding_range_y() }
-}
-
-impl<S: Scalar> MonotonicSegment for LineSegment<S> {
-    type Scalar = S;
-    fn solve_t_for_x(&self, x: S, _t_range: Range<S>, _tolerance: S) -> S {
-        self.solve_t_for_x(x)
+    fn for_each_flattened_with_t(
+        &self,
+        _tolerance: Self::Scalar,
+        callback: &mut dyn FnMut(&LineSegment<S>, Range<S>),
+    ) {
+        callback(self, S::ZERO..S::ONE);
     }
 }
 
-impl<S: Scalar> FlatteningStep for LineSegment<S> {
-    fn flattening_step(&self, _tolerance: S) -> S { S::ONE }
+impl<S: Scalar> BoundingBox for LineSegment<S> {
+    type Scalar = S;
+    fn bounding_range_x(&self) -> (S, S) {
+        self.bounding_range_x()
+    }
+    fn bounding_range_y(&self) -> (S, S) {
+        self.bounding_range_y()
+    }
+    fn fast_bounding_range_x(&self) -> (S, S) {
+        self.bounding_range_x()
+    }
+    fn fast_bounding_range_y(&self) -> (S, S) {
+        self.bounding_range_y()
+    }
 }
 
 /// An infinite line defined by a point and a vector.
@@ -426,24 +585,32 @@ impl<S: Scalar> Line<S> {
         let other_p2 = other.point + other.vector;
         let a = self.point.to_vector().cross(self_p2.to_vector());
         let b = other.point.to_vector().cross(other_p2.to_vector());
-        return Some(
-            point(
-                (b * self.vector.x - a * other.vector.x) * inv_det,
-                (b * self.vector.y - a * other.vector.y) * inv_det,
-            )
-        );
-    }
 
-    pub fn signed_distance_to_point(&self, p: &Point<S>) -> S {
-        let v1 = self.point.to_vector();
-        let v2 = v1 + self.vector;
-        (self.vector.cross(p.to_vector()) + v1.cross(v2)) / self.vector.length()
+        Some(point(
+            (b * self.vector.x - a * other.vector.x) * inv_det,
+            (b * self.vector.y - a * other.vector.y) * inv_det,
+        ))
     }
 
     pub fn distance_to_point(&self, p: &Point<S>) -> S {
         S::abs(self.signed_distance_to_point(p))
     }
 
+    pub fn signed_distance_to_point(&self, p: &Point<S>) -> S {
+        let v = *p - self.point;
+        self.vector.cross(v) / self.vector.length()
+    }
+
+    /// Returned the squared distance to a point.
+    ///
+    /// Can be useful to avoid a square root when comparing against a
+    /// distance that can be squared instead.
+    pub fn square_distance_to_point(&self, p: Point<S>) -> S {
+        let v = p - self.point;
+        let c = self.vector.cross(v);
+        (c * c) / self.vector.square_length()
+    }
+
     pub fn equation(&self) -> LineEquation<S> {
         let a = -self.vector.y;
         let b = self.vector.x;
@@ -451,6 +618,40 @@ impl<S: Scalar> Line<S> {
 
         LineEquation::new(a, b, c)
     }
+
+    pub fn intersects_box(&self, rect: &Box2D<S>) -> bool {
+        let v = self.vector;
+
+        let diagonal = if (v.y >= S::ZERO) ^ (v.x >= S::ZERO) {
+            LineSegment {
+                from: rect.min,
+                to: rect.max,
+            }
+        } else {
+            LineSegment {
+                from: point(rect.max.x, rect.min.y),
+                to: point(rect.min.x, rect.max.y),
+            }
+        };
+
+        diagonal.intersects_line(self)
+    }
+
+    #[inline]
+    pub fn to_f32(&self) -> Line<f32> {
+        Line {
+            point: self.point.to_f32(),
+            vector: self.vector.to_f32(),
+        }
+    }
+
+    #[inline]
+    pub fn to_f64(&self) -> Line<f64> {
+        Line {
+            point: self.point.to_f64(),
+            vector: self.vector.to_f64(),
+        }
+    }
 }
 
 /// A line defined by the equation
@@ -467,17 +668,27 @@ impl<S: Scalar> LineEquation<S> {
     pub fn new(a: S, b: S, c: S) -> Self {
         debug_assert!(a != S::ZERO || b != S::ZERO);
         let div = S::ONE / S::sqrt(a * a + b * b);
-        LineEquation { a: a * div, b: b * div, c: c * div }
+        LineEquation {
+            a: a * div,
+            b: b * div,
+            c: c * div,
+        }
     }
 
     #[inline]
-    pub fn a(&self) -> S { self.a }
+    pub fn a(&self) -> S {
+        self.a
+    }
 
     #[inline]
-    pub fn b(&self) -> S { self.b }
+    pub fn b(&self) -> S {
+        self.b
+    }
 
     #[inline]
-    pub fn c(&self) -> S { self.c }
+    pub fn c(&self) -> S {
+        self.c
+    }
 
     pub fn project_point(&self, p: &Point<S>) -> Point<S> {
         point(
@@ -498,18 +709,30 @@ impl<S: Scalar> LineEquation<S> {
 
     #[inline]
     pub fn invert(&self) -> Self {
-        LineEquation { a: -self.a, b: -self.b, c: -self.c }
+        LineEquation {
+            a: -self.a,
+            b: -self.b,
+            c: -self.c,
+        }
     }
 
     #[inline]
     pub fn parallel_line(&self, p: &Point<S>) -> Self {
         let c = -(self.a * p.x + self.b * p.y);
-        LineEquation { a: self.a, b: self.b, c }
+        LineEquation {
+            a: self.a,
+            b: self.b,
+            c,
+        }
     }
 
     #[inline]
     pub fn offset(&self, d: S) -> Self {
-        LineEquation { a: self.a, b: self.b, c: self.c - d }
+        LineEquation {
+            a: self.a,
+            b: self.b,
+            c: self.c - d,
+        }
     }
 
     #[inline]
@@ -553,7 +776,7 @@ impl<S: Scalar> LineEquation<S> {
 
 #[cfg(test)]
 fn fuzzy_eq_f32(a: f32, b: f32, epsilon: f32) -> bool {
-    return f32::abs(a - b) <= epsilon;
+    f32::abs(a - b) <= epsilon
 }
 
 #[cfg(test)]
@@ -568,7 +791,7 @@ fn fuzzy_eq_point(a: Point<f32>, b: Point<f32>, epsilon: f32) -> bool {
 
 #[test]
 fn intersection_rotated() {
-    use std::f32::consts::PI;
+    use core::f32::consts::PI;
     let epsilon = 0.0001;
     let count: u32 = 100;
 
@@ -594,21 +817,17 @@ fn intersection_rotated() {
 
             assert!(l1.intersects(&l2));
 
-            assert!(
-                fuzzy_eq_point(
-                    l1.sample(l1.intersection_t(&l2).unwrap().0),
-                    point(0.0, 0.0),
-                    epsilon
-                )
-            );
+            assert!(fuzzy_eq_point(
+                l1.sample(l1.intersection_t(&l2).unwrap().0),
+                point(0.0, 0.0),
+                epsilon
+            ));
 
-            assert!(
-                fuzzy_eq_point(
-                    l2.sample(l1.intersection_t(&l2).unwrap().1),
-                    point(0.0, 0.0),
-                    epsilon
-                )
-            );
+            assert!(fuzzy_eq_point(
+                l2.sample(l1.intersection_t(&l2).unwrap().1),
+                point(0.0, 0.0),
+                epsilon
+            ));
         }
     }
 }
@@ -632,7 +851,7 @@ fn intersection_touching() {
 #[test]
 fn intersection_overlap() {
     // It's hard to define the intersection points of two segments that overlap,
-    // (would be a region rather than a point) and more importanly, in practice
+    // (would be a region rather than a point) and more importantly, in practice
     // the algorithms in lyon don't need to consider this special case as an intersection,
     // so we choose to treat overlapping segments as not intersecting.
 
@@ -650,40 +869,47 @@ fn intersection_overlap() {
     assert!(l1.intersection(&l2).is_none());
 }
 
-#[cfg(test)]
-use euclid::rect;
 #[cfg(test)]
 use euclid::approxeq::ApproxEq;
 
 #[test]
-fn bounding_rect() {
+fn bounding_box() {
     let l1 = LineSegment {
-        from: point(1., 5.),
-        to: point(5., 7.),
+        from: point(1.0, 5.0),
+        to: point(5.0, 7.0),
+    };
+    let r1 = Box2D {
+        min: point(1.0, 5.0),
+        max: point(5.0, 7.0),
     };
-    let r1 = rect(1., 5., 4., 2.);
 
     let l2 = LineSegment {
-        from: point(5., 5.),
-        to: point(1., 1.),
+        from: point(5.0, 5.0),
+        to: point(1.0, 1.0),
+    };
+    let r2 = Box2D {
+        min: point(1.0, 1.0),
+        max: point(5.0, 5.0),
     };
-    let r2 = rect(1., 1., 4., 4.);
 
     let l3 = LineSegment {
-        from: point(3., 3.),
-        to: point(1., 5.),
+        from: point(3.0, 3.0),
+        to: point(1.0, 5.0),
+    };
+    let r3 = Box2D {
+        min: point(1.0, 3.0),
+        max: point(3.0, 5.0),
     };
-    let r3 = rect(1., 3., 2., 2.);
 
-    let cases = vec![(l1, r1), (l2, r2), (l3, r3)];
+    let cases = std::vec![(l1, r1), (l2, r2), (l3, r3)];
     for &(ls, r) in &cases {
-        assert_eq!(ls.bounding_rect(), r);
+        assert_eq!(ls.bounding_box(), r);
     }
 }
 
 #[test]
 fn distance_to_point() {
-    use crate::math::vector;
+    use crate::vector;
 
     let l1 = Line {
         point: point(2.0f32, 3.0),
@@ -695,27 +921,58 @@ fn distance_to_point() {
         vector: vector(1.5, 1.5),
     };
 
-    assert!(l1.signed_distance_to_point(&point(1.1, 4.0)).approx_eq(&-1.0));
-    assert!(l1.signed_distance_to_point(&point(2.3, 2.0)).approx_eq(&1.0));
+    assert!(l1
+        .signed_distance_to_point(&point(1.1, 4.0))
+        .approx_eq(&-1.0));
+    assert!(l1
+        .signed_distance_to_point(&point(2.3, 2.0))
+        .approx_eq(&1.0));
 
-    assert!(l2.signed_distance_to_point(&point(1.0, 0.0)).approx_eq(&(-f32::sqrt(2.0)/2.0)));
-    assert!(l2.signed_distance_to_point(&point(0.0, 1.0)).approx_eq(&(f32::sqrt(2.0)/2.0)));
+    assert!(l2
+        .signed_distance_to_point(&point(1.0, 0.0))
+        .approx_eq(&(-f32::sqrt(2.0) / 2.0)));
+    assert!(l2
+        .signed_distance_to_point(&point(0.0, 1.0))
+        .approx_eq(&(f32::sqrt(2.0) / 2.0)));
 
-    assert!(l1.equation().distance_to_point(&point(1.1, 4.0)).approx_eq(&1.0));
-    assert!(l1.equation().distance_to_point(&point(2.3, 2.0)).approx_eq(&1.0));
-    assert!(l2.equation().distance_to_point(&point(1.0, 0.0)).approx_eq(&(f32::sqrt(2.0)/2.0)));
-    assert!(l2.equation().distance_to_point(&point(0.0, 1.0)).approx_eq(&(f32::sqrt(2.0)/2.0)));
+    assert!(l1
+        .equation()
+        .distance_to_point(&point(1.1, 4.0))
+        .approx_eq(&1.0));
+    assert!(l1
+        .equation()
+        .distance_to_point(&point(2.3, 2.0))
+        .approx_eq(&1.0));
+    assert!(l2
+        .equation()
+        .distance_to_point(&point(1.0, 0.0))
+        .approx_eq(&(f32::sqrt(2.0) / 2.0)));
+    assert!(l2
+        .equation()
+        .distance_to_point(&point(0.0, 1.0))
+        .approx_eq(&(f32::sqrt(2.0) / 2.0)));
 
-    assert!(l1.equation().signed_distance_to_point(&point(1.1, 4.0)).approx_eq(&l1.signed_distance_to_point(&point(1.1, 4.0))));
-    assert!(l1.equation().signed_distance_to_point(&point(2.3, 2.0)).approx_eq(&l1.signed_distance_to_point(&point(2.3, 2.0))));
+    assert!(l1
+        .equation()
+        .signed_distance_to_point(&point(1.1, 4.0))
+        .approx_eq(&l1.signed_distance_to_point(&point(1.1, 4.0))));
+    assert!(l1
+        .equation()
+        .signed_distance_to_point(&point(2.3, 2.0))
+        .approx_eq(&l1.signed_distance_to_point(&point(2.3, 2.0))));
 
-    assert!(l2.equation().signed_distance_to_point(&point(1.0, 0.0)).approx_eq(&l2.signed_distance_to_point(&point(1.0, 0.0))));
-    assert!(l2.equation().signed_distance_to_point(&point(0.0, 1.0)).approx_eq(&l2.signed_distance_to_point(&point(0.0, 1.0))));
+    assert!(l2
+        .equation()
+        .signed_distance_to_point(&point(1.0, 0.0))
+        .approx_eq(&l2.signed_distance_to_point(&point(1.0, 0.0))));
+    assert!(l2
+        .equation()
+        .signed_distance_to_point(&point(0.0, 1.0))
+        .approx_eq(&l2.signed_distance_to_point(&point(0.0, 1.0))));
 }
 
 #[test]
 fn solve_y_for_x() {
-
     let line = Line {
         point: Point::new(1.0, 1.0),
         vector: Vector::new(2.0, 4.0),
@@ -723,7 +980,7 @@ fn solve_y_for_x() {
     let eqn = line.equation();
 
     if let Some(y) = eqn.solve_y_for_x(line.point.x) {
-        println!("{:?} != {:?}", y, line.point.y);
+        std::println!("{y:?} != {:?}", line.point.y);
         assert!(f64::abs(y - line.point.y) < 0.000001)
     }
 
@@ -741,7 +998,7 @@ fn solve_y_for_x() {
         let eqn = line.equation();
 
         if let Some(y) = eqn.solve_y_for_x(line.point.x) {
-            println!("{:?} != {:?}", y, line.point.y);
+            std::println!("{y:?} != {:?}", line.point.y);
             assert!(f64::abs(y - line.point.y) < 0.000001)
         }
 
@@ -780,68 +1037,53 @@ fn set_length() {
 
 #[test]
 fn overlap() {
-    assert!(
-        LineSegment {
-            from: point(0.0, 0.0),
-            to: point(-1.0, 0.0),
-        }.overlaps_line(
-            &Line {
-                point: point(100.0, 0.0),
-                vector: vector(10.0, 0.0),
-            }
-        )
-    );
+    assert!(LineSegment {
+        from: point(0.0, 0.0),
+        to: point(-1.0, 0.0),
+    }
+    .overlaps_line(&Line {
+        point: point(100.0, 0.0),
+        vector: vector(10.0, 0.0),
+    }));
 
-    assert!(
-        LineSegment {
-            from: point(0.0, 0.0),
-            to: point(1.0, 0.0),
-        }.overlaps_line(
-            &Line {
-                point: point(0.0, 0.0),
-                vector: vector(1.0, 0.0),
-            }
-        )
-    );
+    assert!(LineSegment {
+        from: point(0.0, 0.0),
+        to: point(1.0, 0.0),
+    }
+    .overlaps_line(&Line {
+        point: point(0.0, 0.0),
+        vector: vector(1.0, 0.0),
+    }));
 
-    assert!(
-        LineSegment {
-            from: point(0.0, 0.0),
-            to: point(1.0, 0.0),
-        }.overlaps_segment(
-            &LineSegment {
-                from: point(0.0, 0.0),
-                to: point(1.0, 0.0),
-            }
-        )
-    );
+    assert!(LineSegment {
+        from: point(0.0, 0.0),
+        to: point(1.0, 0.0),
+    }
+    .overlaps_segment(&LineSegment {
+        from: point(0.0, 0.0),
+        to: point(1.0, 0.0),
+    }));
 
-    assert!(
-        !LineSegment {
-            from: point(0.0, 0.0),
-            to: point(1.0, 0.0),
-        }.overlaps_line(
-            &Line {
-                point: point(0.0, 1.0),
-                vector: vector(1.0, 1.0),
-            }
-        )
-    );
+    assert!(!LineSegment {
+        from: point(0.0, 0.0),
+        to: point(1.0, 0.0),
+    }
+    .overlaps_line(&Line {
+        point: point(0.0, 1.0),
+        vector: vector(1.0, 1.0),
+    }));
 }
 
 #[test]
 fn contains_segment() {
-    assert!(
-        LineSegment {
-            from: point(-1.0, 1.0),
-            to: point(4.0, 1.0),
-        }.contains_segment(
-            &LineSegment {
-                from: point(2.0, 1.0),
-                to: point(1.0, 1.0),
-            }
-        )
-    );
+    assert!(LineSegment {
+        from: point(-1.0, 1.0),
+        to: point(4.0, 1.0),
+    }
+    .contains_segment(&LineSegment {
+        from: point(2.0, 1.0),
+        to: point(1.0, 1.0),
+    }));
 }
 
 #[test]
@@ -874,3 +1116,383 @@ fn horizontal_line_intersection() {
     assert_eq!(segment.horizontal_line_intersection_t(1.5), None);
     assert_eq!(segment.horizontal_line_intersection_t(3.5), None);
 }
+
+#[test]
+fn intersection_on_endpoint() {
+    let l1 = LineSegment {
+        from: point(0.0, 0.0),
+        to: point(0.0, 10.0),
+    };
+
+    let l2 = LineSegment {
+        from: point(0.0, 5.0),
+        to: point(10.0, 5.0),
+    };
+
+    assert_eq!(l1.intersection_t(&l2), Some((0.5, 0.0)));
+    assert_eq!(l2.intersection_t(&l1), Some((0.0, 0.5)));
+
+    let l3 = LineSegment {
+        from: point(10.0, 5.0),
+        to: point(0.0, 5.0),
+    };
+
+    assert_eq!(l1.intersection_t(&l3), Some((0.5, 1.0)));
+    assert_eq!(l3.intersection_t(&l1), Some((1.0, 0.5)));
+}
+
+#[test]
+fn intersects_box() {
+    let b = Box2D {
+        min: point(1.0, 2.0),
+        max: point(4.0, 4.0),
+    };
+
+    assert!(!Line {
+        point: point(0.0, 0.0),
+        vector: vector(1.0, 0.0)
+    }
+    .intersects_box(&b));
+    assert!(!Line {
+        point: point(0.0, 0.0),
+        vector: vector(0.0, 1.0)
+    }
+    .intersects_box(&b));
+    assert!(!Line {
+        point: point(10.0, 0.0),
+        vector: vector(10.0, 10.0)
+    }
+    .intersects_box(&b));
+    assert!(!Line {
+        point: point(0.0, 10.0),
+        vector: vector(10.0, 10.0)
+    }
+    .intersects_box(&b));
+
+    assert!(Line {
+        point: point(1.5, 0.0),
+        vector: vector(1.0, 6.0)
+    }
+    .intersects_box(&b));
+    assert!(Line {
+        point: point(1.5, 0.0),
+        vector: vector(-1.0, 6.0)
+    }
+    .intersects_box(&b));
+    assert!(Line {
+        point: point(1.5, 2.5),
+        vector: vector(1.0, 0.5)
+    }
+    .intersects_box(&b));
+    assert!(Line {
+        point: point(1.5, 2.5),
+        vector: vector(-1.0, -2.0)
+    }
+    .intersects_box(&b));
+}
+
+#[test]
+fn clipped() {
+    let b = Box2D {
+        min: point(1.0, 2.0),
+        max: point(3.0, 4.0),
+    };
+
+    fn approx_eq(a: LineSegment<f32>, b: LineSegment<f32>) -> bool {
+        let ok = a.from.approx_eq(&b.from) && a.to.approx_eq(&b.to);
+        if !ok {
+            std::println!("{a:?} != {b:?}");
+        }
+
+        ok
+    }
+
+    assert_eq!(
+        LineSegment {
+            from: point(0.0, 1.0),
+            to: point(4.0, 1.0)
+        }
+        .clipped(&b),
+        None
+    );
+    assert_eq!(
+        LineSegment {
+            from: point(0.0, 2.0),
+            to: point(4.0, 2.0)
+        }
+        .clipped(&b),
+        Some(LineSegment {
+            from: point(1.0, 2.0),
+            to: point(3.0, 2.0)
+        })
+    );
+    assert_eq!(
+        LineSegment {
+            from: point(0.0, 3.0),
+            to: point(4.0, 3.0)
+        }
+        .clipped(&b),
+        Some(LineSegment {
+            from: point(1.0, 3.0),
+            to: point(3.0, 3.0)
+        })
+    );
+    assert_eq!(
+        LineSegment {
+            from: point(0.0, 4.0),
+            to: point(4.0, 4.0)
+        }
+        .clipped(&b),
+        Some(LineSegment {
+            from: point(1.0, 4.0),
+            to: point(3.0, 4.0)
+        })
+    );
+    assert_eq!(
+        LineSegment {
+            from: point(0.0, 5.0),
+            to: point(4.0, 5.0)
+        }
+        .clipped(&b),
+        None
+    );
+
+    assert_eq!(
+        LineSegment {
+            from: point(4.0, 1.0),
+            to: point(0.0, 1.0)
+        }
+        .clipped(&b),
+        None
+    );
+    assert_eq!(
+        LineSegment {
+            from: point(4.0, 2.0),
+            to: point(0.0, 2.0)
+        }
+        .clipped(&b),
+        Some(LineSegment {
+            from: point(3.0, 2.0),
+            to: point(1.0, 2.0)
+        })
+    );
+    assert_eq!(
+        LineSegment {
+            from: point(4.0, 3.0),
+            to: point(0.0, 3.0)
+        }
+        .clipped(&b),
+        Some(LineSegment {
+            from: point(3.0, 3.0),
+            to: point(1.0, 3.0)
+        })
+    );
+    assert_eq!(
+        LineSegment {
+            from: point(4.0, 4.0),
+            to: point(0.0, 4.0)
+        }
+        .clipped(&b),
+        Some(LineSegment {
+            from: point(3.0, 4.0),
+            to: point(1.0, 4.0)
+        })
+    );
+    assert_eq!(
+        LineSegment {
+            from: point(4.0, 5.0),
+            to: point(0.0, 5.0)
+        }
+        .clipped(&b),
+        None
+    );
+
+    assert_eq!(
+        LineSegment {
+            from: point(0.0, 0.0),
+            to: point(0.0, 5.0)
+        }
+        .clipped(&b),
+        None
+    );
+    assert_eq!(
+        LineSegment {
+            from: point(1.0, 0.0),
+            to: point(1.0, 5.0)
+        }
+        .clipped(&b),
+        Some(LineSegment {
+            from: point(1.0, 2.0),
+            to: point(1.0, 4.0)
+        })
+    );
+    assert_eq!(
+        LineSegment {
+            from: point(2.0, 0.0),
+            to: point(2.0, 5.0)
+        }
+        .clipped(&b),
+        Some(LineSegment {
+            from: point(2.0, 2.0),
+            to: point(2.0, 4.0)
+        })
+    );
+    assert_eq!(
+        LineSegment {
+            from: point(3.0, 0.0),
+            to: point(3.0, 5.0)
+        }
+        .clipped(&b),
+        Some(LineSegment {
+            from: point(3.0, 2.0),
+            to: point(3.0, 4.0)
+        })
+    );
+    assert_eq!(
+        LineSegment {
+            from: point(4.0, 0.0),
+            to: point(4.0, 5.0)
+        }
+        .clipped(&b),
+        None
+    );
+
+    assert_eq!(
+        LineSegment {
+            from: point(0.0, 5.0),
+            to: point(0.0, 0.0)
+        }
+        .clipped(&b),
+        None
+    );
+    assert_eq!(
+        LineSegment {
+            from: point(1.0, 5.0),
+            to: point(1.0, 0.0)
+        }
+        .clipped(&b),
+        Some(LineSegment {
+            from: point(1.0, 4.0),
+            to: point(1.0, 2.0)
+        })
+    );
+    assert_eq!(
+        LineSegment {
+            from: point(2.0, 5.0),
+            to: point(2.0, 0.0)
+        }
+        .clipped(&b),
+        Some(LineSegment {
+            from: point(2.0, 4.0),
+            to: point(2.0, 2.0)
+        })
+    );
+    assert_eq!(
+        LineSegment {
+            from: point(3.0, 5.0),
+            to: point(3.0, 0.0)
+        }
+        .clipped(&b),
+        Some(LineSegment {
+            from: point(3.0, 4.0),
+            to: point(3.0, 2.0)
+        })
+    );
+    assert_eq!(
+        LineSegment {
+            from: point(4.0, 5.0),
+            to: point(4.0, 0.0)
+        }
+        .clipped(&b),
+        None
+    );
+
+    assert!(approx_eq(
+        LineSegment {
+            from: point(0.0, 2.0),
+            to: point(4.0, 4.0)
+        }
+        .clipped(&b)
+        .unwrap(),
+        LineSegment {
+            from: point(1.0, 2.5),
+            to: point(3.0, 3.5)
+        }
+    ));
+    assert!(approx_eq(
+        LineSegment {
+            from: point(4.0, 4.0),
+            to: point(0.0, 2.0)
+        }
+        .clipped(&b)
+        .unwrap(),
+        LineSegment {
+            from: point(3.0, 3.5),
+            to: point(1.0, 2.5)
+        }
+    ));
+
+    let inside = [
+        LineSegment {
+            from: point(1.0, 2.0),
+            to: point(3.0, 4.0),
+        },
+        LineSegment {
+            from: point(1.5, 2.0),
+            to: point(1.0, 4.0),
+        },
+        LineSegment {
+            from: point(1.0, 3.0),
+            to: point(2.0, 3.0),
+        },
+    ];
+
+    for segment in &inside {
+        assert_eq!(segment.clipped(&b), Some(*segment));
+        assert_eq!(segment.flip().clipped(&b), Some(segment.flip()));
+    }
+
+    let outside = [
+        LineSegment {
+            from: point(2.0, 0.0),
+            to: point(5.0, 3.0),
+        },
+        LineSegment {
+            from: point(-20.0, 0.0),
+            to: point(4.0, 8.0),
+        },
+    ];
+
+    for segment in &outside {
+        assert_eq!(segment.clipped(&b), None);
+        assert_eq!(segment.flip().clipped(&b), None);
+    }
+}
+
+#[test]
+fn equation() {
+    let lines = [
+        Line {
+            point: point(100.0f64, 20.0),
+            vector: vector(-1.0, 3.0),
+        },
+        Line {
+            point: point(-30.0, 150.0),
+            vector: vector(10.0, 2.0),
+        },
+        Line {
+            point: point(50.0, -10.0),
+            vector: vector(5.0, -1.0),
+        },
+    ];
+
+    for line in &lines {
+        let eqn = line.equation();
+        use euclid::approxeq::ApproxEq;
+        for t in [-100.0, -50.0, 0.0, 25.0, 325.0] {
+            let p = line.point + line.vector * t;
+            assert!(eqn.solve_y_for_x(p.x).unwrap().approx_eq(&p.y));
+            assert!(eqn.solve_x_for_y(p.y).unwrap().approx_eq(&p.x));
+        }
+    }
+}
